
From: "Andi Kleen" <ak@suse.de>

To avoid some code duplication.

Signed-off-by: Andi Kleen <ak@suse.de>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/x86_64/mm/fault.c      |   12 +++++-------
 25-akpm/include/asm-x86_64/ptrace.h |    3 +++
 2 files changed, 8 insertions(+), 7 deletions(-)

diff -puN arch/x86_64/mm/fault.c~x86_64-use-a-common-function-to-find-code-segment-bases arch/x86_64/mm/fault.c
--- 25/arch/x86_64/mm/fault.c~x86_64-use-a-common-function-to-find-code-segment-bases	Thu Apr  7 15:10:45 2005
+++ 25-akpm/arch/x86_64/mm/fault.c	Thu Apr  7 15:10:45 2005
@@ -62,21 +62,19 @@ void bust_spinlocks(int yes)
 static noinline int is_prefetch(struct pt_regs *regs, unsigned long addr,
 				unsigned long error_code)
 { 
-	unsigned char *instr = (unsigned char *)(regs->rip);
+	unsigned char *instr;
 	int scan_more = 1;
 	int prefetch = 0; 
-	unsigned char *max_instr = instr + 15;
+	unsigned char *max_instr;
 
 	/* If it was a exec fault ignore */
 	if (error_code & (1<<4))
 		return 0;
 	
-	/* Code segments in LDT could have a non zero base. Don't check
-	   when that's possible */
-	if (regs->cs & (1<<2))
-		return 0;
+	instr = (unsigned char *)convert_rip_to_linear(current, regs);
+	max_instr = instr + 15;
 
-	if ((regs->cs & 3) != 0 && regs->rip >= TASK_SIZE)
+	if ((regs->cs & 3) != 0 && instr >= (unsigned char *)TASK_SIZE)
 		return 0;
 
 	while (scan_more && instr < max_instr) { 
diff -puN include/asm-x86_64/ptrace.h~x86_64-use-a-common-function-to-find-code-segment-bases include/asm-x86_64/ptrace.h
--- 25/include/asm-x86_64/ptrace.h~x86_64-use-a-common-function-to-find-code-segment-bases	Thu Apr  7 15:10:45 2005
+++ 25-akpm/include/asm-x86_64/ptrace.h	Thu Apr  7 15:10:45 2005
@@ -86,6 +86,9 @@ struct pt_regs {
 extern unsigned long profile_pc(struct pt_regs *regs);
 void signal_fault(struct pt_regs *regs, void __user *frame, char *where);
 
+extern unsigned long
+convert_rip_to_linear(struct task_struct *child, struct pt_regs *regs);
+
 enum {
         EF_CF   = 0x00000001,
         EF_PF   = 0x00000004,
_
